// *****************************************************************************
// * Copyright (c) 1999      Southeastern Universities Research Association,
// *                         Thomas Jefferson National Accelerator Facility
// * 
// * This software was developed under a United States Government license
// * described in the NOTICE file included as part of this distribution.
// *****************************************************************************
// *
// * File:        PBS_Base.h
// * Author:      Walt Akers
// * Description: This file contains the class definition for the PBS_Base
// *              class.  This class provides mechanisms that are generally 
// *              useful to the other PBS classes.
// *****************************************************************************
#include "PBS_Base.h"

// *********************************************************************
// * PBSStringIndex queueHash :
// *	This hash table is used to associate queueName with integer 
// *	identifiers.  The next available queue ID is maintained in the
// *	nextQueueID variable.
// *********************************************************************
PBSStringIndex PBS_Base::queueHash   (16, 1);
unsigned       PBS_Base::nextQueueID (1);

// *********************************************************************
// * PBSStringIndex userHash :
// *	This has table is used to associate a userName with an integer
// *	identifier. The next available user ID is maintained in the
// *	nextUserID variable.
// *********************************************************************
PBSStringIndex PBS_Base::userHash    (256, 1);
unsigned       PBS_Base::nextUserID  (1);


// *********************************************************************
// * PBS_Base::getUserIDFromName :
// *	This method allows the caller to obtain the userID associated
// *	with the user name.  If the name has not been defined, then it
// *	will be automatically created and added to the hash table.
// *********************************************************************
unsigned PBS_Base::getUserIDFromName ( const char * name )
	{
	static char localUser[256];
	size_t idx = 0;

	for(idx=0; idx<255 && name && name[idx] && name[idx]!='@'; idx++);
	if(idx>0) 
		{
		strncpy(localUser, name, idx);
		localUser[idx] = 0;
		}
	else	{
		strcpy(localUser, "unknown");
		}

	unsigned userID = userHash.find(localUser);
	if(!userID)
		{
		userID = nextUserID++;
		userHash.insert(localUser, userID);
		}

	return userID;
	}
	
// *********************************************************************
// * PBS_Base::getQueueIDFromName :
// *	This method allows the caller to obtain the queueID associated
// *	with the user name.  If the name has not been defined, then it
// *	will be automatically created and added to the hash table.
// *********************************************************************
unsigned PBS_Base::getQueueIDFromName ( const char * queueName )
	{
	unsigned queueID = queueHash.find(queueName);
	if(!queueID)
		{
		queueID = nextQueueID++;
		queueHash.insert(queueName, queueID);
		}
	return queueID;
	}
