#include <PBS_Server.h>
const char * msg_daemonname = SetLogLocal("XMLDump");

void generateXMLDump (PBS_Server & server );

int main ( int argc, char ** argv)
{
if(argc==1)
	{
	fprintf(stderr, "Format is: XMLDump [server] <server> <...>\n");
	return (-1);
	}

char timeStr[64];
time_t currTime = time(NULL);
struct tm * currTm = localtime(&currTime);

strftime(timeStr, 64, "%B %d, %Y %H:%M:%S", currTm);
fprintf(stdout, "<servers>\n");
fprintf(stdout, "<generation_time>%s</generation_time>\n", timeStr);
for(int x=1; x<argc; x++)
	{
	int connID = pbs_connect(argv[x]);
	if(connID>0)
		{
		PBS_Server server(connID);
		generateXMLDump(server);
		}
	pbs_disconnect(connID);
	}
fprintf(stdout, "</servers>\n");
return 0;
}


void generateXMLDump ( PBS_Server & server )
{
size_t idx;

fprintf(stdout, "<server>\n");
fprintf(stdout, "\t<name>%s</name>\n", server.getName());	
fprintf(stdout, "\t<default_cpu_time>%s</default_cpu_time>\n", server.getDefaultResources()->find("cput"));
fprintf(stdout, "\t<default_file_size>%s</default_file_size>\n", server.getDefaultResources()->find("file"));
fprintf(stdout, "\t<default_walltime>%s</default_walltime>\n", server.getDefaultResources()->find("walltime"));
fprintf(stdout, "\t<maximum_cpu_time>%s</maximum_cpu_time>\n", server.getMaxResources()->find("cput"));
fprintf(stdout, "\t<maximum_file_size>%s</maximum_file_size>\n", server.getMaxResources()->find("file"));
fprintf(stdout, "\t<maximum_walltime>%s</maximum_walltime>\n", server.getMaxResources()->find("walltime"));	
fprintf(stdout, "\t<scheduling>%s</scheduling>\n", server.getSchedulingFlag()?"yes":"no");	
fprintf(stdout, "\t<total_queue>%i</total_queue>\n", server.getQueueCount());
fprintf(stdout, "\t<default_queue>%s</default_queue>\n", server.getDefaultQueueName());

PBS_JobList   & jList = server;
PBS_QueueList & qList = server;
PBS_NodeList  & nList = server;
PBS_Queue     * queue;
PBS_Node      * node;

for(idx=0; (queue=qList[idx])!=NULL; idx++)
	{
	fprintf(stdout, "\t<queue>\n");
	fprintf(stdout, "\t\t<name>%s</name>\n", queue->getName());
	fprintf(stdout, "\t\t<default_cpu_time>%s</default_cpu_time>\n", queue->getDefaultResources()->find("cput"));
	fprintf(stdout, "\t\t<default_file_size>%s</default_file_size>\n", queue->getDefaultResources()->find("file"));
	fprintf(stdout, "\t\t<default_walltime>%s</default_walltime>\n", queue->getDefaultResources()->find("walltime"));
	fprintf(stdout, "\t\t<maximum_cpu_time>%s</maximum_cpu_time>\n", queue->getMaxResources()->find("cput"));
	fprintf(stdout, "\t\t<maximum_file_size>%s</maximum_file_size>\n", queue->getMaxResources()->find("file"));
	fprintf(stdout, "\t\t<maximum_walltime>%s</maximum_walltime>\n", queue->getMaxResources()->find("walltime"));	
	fprintf(stdout, "\t\t<total_jobs>%i</total_jobs>\n", queue->getTotalJobCount());
	fprintf(stdout, "\t\t<transit_jobs>%i</transit_jobs>\n", queue->getTransitJobCount());
	fprintf(stdout, "\t\t<queued_jobs>%i</queued_jobs>\n", queue->getQueuedJobCount());
	fprintf(stdout, "\t\t<held_jobs>%i</held_jobs>\n", queue->getHeldJobCount());
	fprintf(stdout, "\t\t<waiting_jobs>%i</waiting_jobs>\n", queue->getWaitingJobCount());
	fprintf(stdout, "\t\t<running_jobs>%i</running_jobs>\n", queue->getRunningJobCount());
	PBS_Job ** jobs = jList.getJobsByQueue((char *)queue->getName());
	for(int jIdx=0; jobs && jobs[jIdx]!=NULL; jIdx++)
		{
		fprintf(stdout, "\t\t<job job_id=\"%s\">\n", jobs[jIdx]->getName());
		fprintf(stdout, "\t\t\t<name>%s</name>\n", jobs[jIdx]->getUserJobName());
		fprintf(stdout, "\t\t\t<state>%s</state>\n", jobs[jIdx]->getStateName());
		fprintf(stdout, "\t\t\t<owner>%s</owner>\n", jobs[jIdx]->getOwner());
		fprintf(stdout, "\t\t\t<cpu_time>%s</cpu_time>\n", jobs[jIdx]->getUsedResources()->find("cput"));
		fprintf(stdout, "\t\t\t<walltime>%s</walltime>\n", jobs[jIdx]->getUsedResources()->find("walltime"));

		const char ** hosts = jobs[jIdx]->getExecutionHosts();
		for(int hIdx=0; hosts && hosts[hIdx]; hIdx++)
			{
			fprintf(stdout, "\t\t\t<executing_on>%s</executing_on>\n", hosts[hIdx]);
			}
		fprintf(stdout, "\t\t</job>\n");
		}	
	fprintf(stdout, "\t</queue>\n");
	delete [] jobs;
	}

fprintf(stdout, "\t<nodes>\n");
for(idx=0; (node=nList[idx])!=NULL; idx++)
	{
	fprintf(stdout, "\t\t<node>\n");
	fprintf(stdout, "\t\t\t<name>%s</name>\n", node->getName());
	fprintf(stdout, "\t\t\t<cpu>%i</cpu>\n", node->getNumProcessors());
	fprintf(stdout, "\t\t\t<state>%s</state>\n", node->getStateName());

	const char ** keywords = node->getKeywords();
	for(int kIdx=0; keywords && keywords[kIdx] && *keywords[kIdx]; kIdx++)
		{
		fprintf(stdout, "\t\t\t<keyword>%s</keyword>\n", keywords[kIdx]);
		}
		
	const char ** jobs = node->getJobs();
	for(int jIdx=0; jobs && jobs[jIdx]!=NULL; jIdx++)
		{
		fprintf(stdout, "\t\t\t<job_id>%s</job_id>\n", jobs[jIdx]);
		}
	fprintf(stdout, "\t\t</node>\n");
	}
fprintf(stdout, "\t</nodes>\n");
fprintf(stdout, "</server>\n");
}
