// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
// 	This header file defines the UserPriorityStage class which computes the
//	weight for a job based on the user specified priority.
//
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _USER_PRIORITY_STAGE_H_
#define _USER_PRIORITY_STAGE_H_ 1

#include <UnderLordStage.h>
#include <PBS_Parameters.h>
#include <PBS_Job.h>

// *****************************************************************************
// * class UserPriorityStage:
// *	This class will compute the weight for a specific job based on the 
// *	historical system utilization by the owner.  Unlike the other stages,
// *	this stage must presort the jobs because the weight assigned for each
// *	user decreases as a job is scheduled.
// *****************************************************************************
class UserPriorityStage : public UnderLordStage
{
public:
	inline UserPriorityStage  ( void );
	inline double getWeight   ( void );
	void          compute     ( PBS_Job ** jobs, size_t jobCnt, double weightSum );
};

// *****************************************************************************
// * UserPriorityStage::UserPriorityStage :
// *	This is the constructor for the UserPriorityStage.
// *****************************************************************************
inline UserPriorityStage::UserPriorityStage ( void )
	: UnderLordStage()
	{
	} 

// *****************************************************************************
// * UserPriorityStage::getWeight :
// *	This method will read from the scheduler profile and will return the
// *	weight associated with this scheduling stage. A default weight of 1.0
// *	will be returned if the parameter has not been specified in the
// *	scheduler profile.
// *****************************************************************************
double UserPriorityStage::getWeight ( void )
	{
	PBS_Parameters & params = PBS_Parameters::attach();
	double weight = params.getConfigFloat("User Priority Stage", "weight", 1.0);
	return (weight<=0.0?0.0:weight);
	}

#endif /* _USER_PRIORITY_STAGE_H_ */
